package org.jboss.messaging.core.impl;

import java.util.Collections;
import java.util.List;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentMap;
import java.util.concurrent.atomic.AtomicLong;

import org.jboss.messaging.core.contract.MessageReference;
import org.jboss.messaging.core.contract.PersistenceManager;
import org.jboss.messaging.core.impl.tx.Transaction;

/**
 * 
 * A NullPersistenceManager
 * 
 * @author <a href="mailto:tim.fox@jboss.com">Tim Fox</a>
 *
 */
public class NullPersistenceManager implements PersistenceManager
{
	private ConcurrentMap<String, AtomicLong> counters = new ConcurrentHashMap<String, AtomicLong>();

	public void addReference(long channelID, MessageReference ref, Transaction tx)
			throws Exception
	{
		//NOOP
	}

	public List getMessageChannelPairAcksForTx(long transactionId)
			throws Exception
	{
		return Collections.EMPTY_LIST;
	}

	public List getMessageChannelPairRefsForTx(long transactionId)
			throws Exception
	{
		return Collections.EMPTY_LIST;
	}

	public List getMessages(List messageIds) throws Exception
	{
		return Collections.EMPTY_LIST;
	}

	public List getPagedReferenceInfos(long channelID, long orderStart,
			int number) throws Exception
	{
		return Collections.EMPTY_LIST;
	}

	public boolean idExists(String messageID) throws Exception
	{		
		return false;
	}

	public InitialLoadInfo loadFromStart(long channelID, int fullSize)
			throws Exception
	{
		return new InitialLoadInfo(null, null, Collections.EMPTY_LIST);
	}

	public InitialLoadInfo mergeAndLoad(long fromChannelID, long toChannelID,
			int numberToLoad, long firstPagingOrder, long nextPagingOrder)
			throws Exception
	{
		return new InitialLoadInfo(null, null, Collections.EMPTY_LIST);
	}

	public void mergeTransactions(int fromNodeID, int toNodeID) throws Exception
	{		
		//NOOP
	}

	public void moveReference(long sourceChannelID, long destChannelID,
			MessageReference ref) throws Exception
	{		
		//NOOP
	}

	public void pageReferences(long channelID, List references, boolean paged)
			throws Exception
	{
		//NOOP
	}

	public void removeDepagedReferences(long channelID, List refs)
			throws Exception
	{
		//NOOP
	}

	public void removeReference(long channelID, MessageReference ref,
			Transaction tx) throws Exception
	{
		//NOOP
	}
	
	public long reserveIDBlock(String counterName, int size) throws Exception
	{		
		AtomicLong counter = counters.get(counterName);
		
		if (counter == null)
		{
			counter = new AtomicLong(0);
			
			AtomicLong oldCounter = counters.putIfAbsent(counterName, counter);
			
			if (oldCounter != null)
			{
				counter = oldCounter;
			}
		}
		
		long idStart = counter.addAndGet(size) - size;
		
		return idStart;
	}

	public List retrievePreparedTransactions() throws Exception
	{
		return Collections.EMPTY_LIST;
	}

	public void updateDeliveryCount(long channelID, MessageReference ref)
			throws Exception
	{
		//NOOP
	}

	public void updatePageOrder(long channelID, List references)
			throws Exception
	{
		///NOOP
	}

	public void updateReferencesNotPagedInRange(long channelID, long orderStart,
			long orderEnd, long num) throws Exception
	{
		//NOOP
	}

	public void start() throws Exception
	{
		//NOOP
	}

	public void stop() throws Exception
	{
		//NOOP
	}

}
